var api = JIRA.API;
var projectKey;
if (api != undefined) {
    projectKey = api.Projects.getCurrentProjectKey()
}

var baseUrl = AJS.$("meta[name='ajs-base-url']").attr("content");

var reviewIdReviewSummary = new Map();
var openedReviewSummary;
var openedDelinkReviewElement;

document.addEventListener("review-context-updated", function (e) {
    reviewIdReviewSummary = new Map();
    openedReviewSummary = null;
    openedDelinkReviewElement = null;

    var reviewIdColumns = AJS.$(".review-id");
    var maxWidth = 0;

    AJS.$.each(reviewIdColumns, function (index, reviewIdColumn) {
        var reviewIdWidth = reviewIdColumn.offsetWidth;
        if (reviewIdWidth > maxWidth) {
            maxWidth = reviewIdWidth;
        }
    });

    reviewIdColumns.css('width', maxWidth);
});

AJS.$(document).ready(function () {

    AJS.$("body").on("click", ".review", (function (event) {
        AJS.$(".aui-message").remove();
        var reviewElement = AJS.$(this);
        var reviewId = reviewElement.attr("id");

        var reviewSummaryElement = AJS.$("#reviewSummary" + reviewId);
        if (reviewSummaryElement[0].style.display === "block") {
            closeOpenedReviewSummary();
            return;
        }

        closeOpenedReviewSummary();
        openedDelinkReviewElement = reviewElement.find(".delink-review");

        openedReviewSummary = reviewSummaryElement;
        if (reviewIdReviewSummary.get(reviewId)) {
            openReviewSummary();
            return;

        }

        getReviewSummary();

        function openReviewSummary() {
            AJS.$(function () {
                AJS.$(".delink-review").tooltip({gravity: 'ne'});

            });
            var reviewPhase = reviewElement.find(".review-phase")[0].textContent;
            if (isParticipant() && (reviewPhase === 'INSPECTING' || reviewPhase === 'PLANNING'
                || reviewPhase === 'ANNOTATING' || reviewPhase === 'REWORK')) {
                reviewElement.find(".delink-review")[0].style.display = "block";
            }
            reviewSummaryElement[0].style.display = "block";

        }

        function closeOpenedReviewSummary() {
            if (openedReviewSummary) {
                openedReviewSummary[0].style.display = "none";
            }
            if (openedDelinkReviewElement) {
                openedDelinkReviewElement[0].style.display = "none";
            }
        }

        function isParticipant() {
            return (reviewIdReviewSummary.get(reviewId).reviewParticipants
                    .map(participant => participant.user.id)
                    .includes(+user.collaboratorId) || user.admin === 'true')
                && user.loggedIn === "true";
        }

        function getReviewSummary() {
            AJS.$.ajax({
                type: "GET",
                url: baseUrl + "/rest/collab/1.0/reviews/" + reviewId,
                headers: {
                    "projectKey": projectKey
                },
                dataType: "json",
                cache: false,
                timeout: 600000,
                success: success,
                error: error
            })

        }

        function success(reviewSummaryResponse) {
            if (reviewIdReviewSummary.get(reviewId)) {
                return;
            }
            reviewIdReviewSummary.set(reviewId, reviewSummaryResponse);
            reviewSummaryResponse.reviewParticipants.forEach(addParticipantRow);
            reviewSummaryResponse.remoteSystemItems.forEach(addRemoteSystem);
            if (reviewSummaryElement === openedReviewSummary) {
                openReviewSummary();
            }

            function addParticipantRow(participant) {
                var displayName = participant.displayName;

                var role = participant.role.name;
                var participantRow = AJS.$(".participant-header").clone(true);
                participantRow.removeClass("participant-header");
                participantRow.addClass("participant-row");
                participantRow.children(".participant")[0].textContent = displayName;
                participantRow.children(".role")[0].textContent = role;
                reviewSummaryElement.children(".participants")[0].append(participantRow[0]);

            }

            function addRemoteSystem(remoteSystemItem) {
                var remoteSystemUrl = remoteSystemItem.url;
                if (remoteSystemUrl === '') {
                    remoteSystemUrl = remoteSystemItem.title.split('"')[1];
                }


                reviewSummaryElement.children("p").after("<div class='remote-system-link'><a target='_blank' href='" + remoteSystemUrl + "'>" + remoteSystemUrl + "</a></div>")
            }

        }

        function error(e) {
            AJS.messages.error('#reviewMessage' + reviewId, {
                body: '<p> ' + e.responseJSON.message + '</p>'
            });
            console.log(e);
        }

    }));

    AJS.$("body").on("click", "span.delink-review", function (event) {
        var reviewId = AJS.$(this).closest(".review")[0].id;
        var remoteSystemItemId;
        reviewIdReviewSummary.get(reviewId).remoteSystemItems.forEach(function (remoteSystemItem) {
            if (remoteSystemItem.url.includes("/browse/" + JIRA.Issue.getIssueKey())) {
                remoteSystemItemId = remoteSystemItem.id;
            }
        });

        AJS.$("#dialog-section").html(
            "<section id='delink-review-dialog' class='aui-dialog2 aui-dialog2-small aui-layer' data-aui-remove-on-hide=true role='dialog' aria-hidden='true'>\n" +

            "    <header class='aui-dialog2-header'>\n" +
            "        <h2 class='aui-dialog2-header-main'>\n" +
            "           Delete link to the review" +
            "        </h2>\n" +
            "        <a class='aui-dialog2-header-close'>\n" +
            "            <span class='aui-icon aui-icon-small aui-iconfont-close-dialog'>Close</span>\n" +
            "        </a>\n" +
            "    </header>\n" +

            "    <div class='aui-dialog2-content'>\n" +
            "       <h3 style='padding: 30px 0 30px 0'>" +
            "          Do you want to delete link to review #" + reviewId + "?" +
            "       </h3>\n" +
            "    </div>\n" +

            "    <div id='aui-message-bar' style='max-width: 360px; margin:20px 20px 0 20px'></div>\n" +

            "    <footer class='aui-dialog2-footer'>\n" +
            "        <div class='aui-dialog2-footer-actions'>\n" +
            "            <button id='delink-review-button' class='aui-button aui-button-primary' onclick='delinkReview(" + remoteSystemItemId + ")'>\n" +
            "                Delete\n" +
            "            </button>\n" +
            "           <button class='aui-button aui-button-link' onclick='AJS.dialog2(\"#delink-review-dialog\").hide();'>Cancel</button>" +
            "        </div>\n" +
            "    </footer>\n" +

            "</section>");

        AJS.dialog2("#delink-review-dialog").show();
    });
});

function showLoginDialog() {

    AJS.$("#dialog-section").html(
        "<section id='login-dialog' class='aui-dialog2 aui-dialog2-small aui-layer' data-aui-remove-on-hide=true role='dialog' aria-hidden='true'>\n" +

        "    <header class='aui-dialog2-header' style='margin-bottom:10px'>\n" +
        "        <h2 class='aui-dialog2-header-main'>\n" +
        "            <img src=" + baseUrl + "/download/resources/com.smartbear.collab-jira-server-plugin:collab-plugin-resources/images/logo-128.png" +
        "                 width='24' height='24'/> Sign in to Collaborator\n" +
        "        </h2>\n" +
        "        <a class='aui-dialog2-header-close'>\n" +
        "            <span class='aui-icon aui-icon-small aui-iconfont-close-dialog'>Close</span>\n" +
        "        </a>\n" +
        "    </header>\n" +

        "    <div id='aui-message-bar' style='max-width: 360px; margin:20px 20px 0 20px'></div>\n" +

        "    <div class='aui-dialog2-content' style='padding: 20px 0 40px 0'>\n" +
        "        <form id='credentials' action='login' method='post' class='aui'\n" +
        "              style='float:none; display: inline-block; text-align: center;'>\n" +
        "            <div class='field-group'>\n" +
        "                <label for='username'>Username: </label>\n" +
        "                <input type='text' id='username' class='text'>\n" +
        "            </div>\n" +
        "            <div class='field-group'>\n" +
        "                <label for='password'>Password: </label>\n" +
        "                <input type='password' name='password' id='password' class='text'\n" +
        "                       placeholder='enter password'>\n" +
        "            </div>\n" +
        "            <input type='hidden' id='projectKey'/>\n" +
        "        </form>\n" +
        "        <div id='sign-in-problem'></div>\n" +
        "    </div>\n" +

        "    <footer class='aui-dialog2-footer'>\n" +
        "        <div class='aui-dialog2-footer-actions'>\n" +
        "            <button id='sign-in-button' class='aui-button aui-button-primary' onclick='loginUser()'>\n" +
        "                Sign in\n" +
        "            </button>\n" +
        "           <button class='aui-button aui-button-link' onclick='AJS.dialog2(\"#login-dialog\").hide();'>Cancel</button>" +
        "        </div>\n" +
        "    </footer>\n" +

        "</section>");

    AJS.dialog2("#login-dialog").show();
}

function loginUser() {
    AJS.$("#sign-in-button").prop("disabled", true);
    AJS.$(".aui-message").remove();
    var user = {
        login: AJS.$("#username").val(),
        password: window.btoa(AJS.$("#password").val())
    };

    AJS.$.ajax({
        type: "POST",
        contentType: "application/json",
        url: baseUrl + "/rest/collab/1.0/auth/login/" + projectKey,
        data: JSON.stringify(user),
        dataType: "json",
        cache: false,
        timeout: 600000,
        success: success,
        error: error
    });

    function success() {
        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
        AJS.dialog2("#login-dialog").hide();
    }

    function error(e) {
        AJS.$("#sign-in-button").prop("disabled", false);
        AJS.messages.error({
            body: '<p> ' + e.responseText + '</p>'
        });
        console.log(e);
    }
}

function logout() {

    AJS.$.ajax({
        type: "POST",
        contentType: "application/json",
        url: baseUrl + "/rest/collab/1.0/auth/logout/" + projectKey,
        dataType: "json",
        cache: false,
        timeout: 600000,
        success: success,
        error: error
    });

    function success() {
        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
    }

    function error(e) {
        console.log(e);
    }
}

/*function showNewReviewDialog() {

    AJS.$("#dialog-section").html(
        "<section id='new-review-dialog' class='aui-dialog2 aui-dialog2-small aui-layer' data-aui-remove-on-hide=true role='dialog' aria-hidden='true'>\n" +

        "    <header class='aui-dialog2-header'>\n" +
        "        <h2 class='aui-dialog2-header-main'>\n" +
        "           Create new review\n" +
        "        </h2>\n" +
        "        <a class='aui-dialog2-header-close'>\n" +
        "            <span class='aui-icon aui-icon-small aui-iconfont-close-dialog'>Close</span>\n" +
        "        </a>\n" +
        "    </header>\n" +

        "    <div class='aui-dialog2-content' style='padding-top: 40px'>\n" +
        "       <h3 id='message' class='aui-content'>Do you want to create a new review for this issue?</h3>\n" +
        "    </div>\n" +

        "    <div id='aui-message-bar' style='max-width: 360px; margin:20px 20px 0 20px'></div>\n" +

        "    <footer class='aui-dialog2-footer'>\n" +
        "        <div class='aui-dialog2-footer-actions'>\n" +
        "            <button id='credentials-submit-button' class='aui-button aui-button-primary' onclick='createNewReview()'>\n" +
        "                Create\n" +
        "            </button>\n" +
        "           <button class='aui-button aui-button-link' onclick='AJS.dialog2(\"#new-review-dialog\").hide();'>Cancel</button>" +
        "        </div>\n" +
        "    </footer>\n" +

        "</section>");

    AJS.dialog2("#new-review-dialog").show();
}*/

function createNewReview() {
    AJS.$(".aui-message").remove();
    AJS.$.ajax({
        type: "POST",
        contentType: "application/json",
        url: baseUrl + "/rest/collab/1.0/reviews/new",
        headers: {
            "projectKey": projectKey,
            "issueKey": JIRA.Issue.getIssueKey()
        },
        dataType: "json",
        cache: false,
        timeout: 600000,
        success: success,
        error: error
    });

    function success(data, textStatus, request) {
        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
        /*AJS.dialog2("#new-review-dialog").hide();*/
        window.open(request.getResponseHeader('location'), '_blank');
    }

    function error(e) {
        AJS.messages.error('#createReviewMessage', {
            body: '<p> ' + e.responseJSON.message + '</p>'
        });
        console.log(e);
    }
}

function showLinkExistingReviewDialog() {

    AJS.$("#dialog-section").html(
        "<section id='link-existing-review-dialog' class='aui-dialog2 aui-dialog2-medium aui-layer' " +
        "data-aui-remove-on-hide=true role='dialog' aria-hidden='true'>\n" +

        "    <header class='aui-dialog2-header'>\n" +
        "        <h2 class='aui-dialog2-header-main'>\n" +
        "            Link Collaborator Review\n" +
        "        </h2>\n" +
        "        <a class='aui-dialog2-header-close'>\n" +
        "            <span class='aui-icon aui-icon-small aui-iconfont-close-dialog'>Close</span>\n" +
        "        </a>\n" +
        "    </header>\n" +

        "    <div class='aui-dialog2-content'>\n" +
        "        <h3>Link an existing Collaborator review to this issue</h3>" +
        "        <form class='aui'>\n" +
        "            <label for='reviews'>Select Review(s)</label>\n" +
        "            <select class='select' style='max-width: 532px' id='reviews'>\n" +
        usersActiveReviews.map(userActiveReview =>
            "<option value='" + userActiveReview.reviewId + "'>" + userActiveReview.reviewText + '</option>')
            .join("\n") +
        "            </select>\n" +
        "        </form>\n" +
        "        <div id='aui-message-bar' style='margin:20px 20px 0 0; max-width: 532px;'></div>\n" +
        "    </div>\n" +

        "    <footer class='aui-dialog2-footer'>\n" +
        "        <div class='aui-dialog2-footer-actions'>\n" +
        "            <button id='link-review-submit-button' class='aui-button aui-button-primary' onclick='linkReview()'>\n" +
        "                Add\n" +
        "            </button>\n" +
        "           <button class='aui-button aui-button-link' onclick='AJS.dialog2(\"#link-existing-review-dialog\").hide();'>Cancel</button>" +
        "        </div>\n" +
        "    </footer>\n" +

        "</section>");

    AJS.dialog2("#link-existing-review-dialog").show();
}

function linkReview() {
    AJS.$(".aui-message").remove();
    var reviewId = AJS.$('select#reviews').children("option:selected").val();
    if (reviewId === undefined) {
        AJS.messages.error({
            body: '<p> No reviews to link</p>'
        });
    }

    AJS.$.ajax({
        type: "PUT",
        contentType: "application/json",
        url: baseUrl + "/rest/collab/1.0/reviews/" + reviewId + "/issues/" + JIRA.Issue.getIssueKey(),
        headers: {
            "projectKey": projectKey
        },
        dataType: "json",
        cache: false,
        timeout: 600000,
        success: success,
        error: showDefaultErrorMessage
    });

    function success() {
        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
        AJS.dialog2("#link-existing-review-dialog").hide();
    }
}

function delinkReview(remoteSystemItemId) {
    AJS.$(".aui-message").remove();
    AJS.$.ajax({
        type: "DELETE",
        url: baseUrl + "/rest/collab/1.0/reviews/remote-system-link/" + remoteSystemItemId,
        headers: {
            "projectKey": projectKey
        },
        dataType: "json",
        cache: false,
        timeout: 600000,
        success: success,
        error: showDefaultErrorMessage
    });

    function success(data) {
        console.log(data);
        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
        AJS.dialog2("#delink-review-dialog").hide();
    }
}

function showDefaultErrorMessage(e) {
    AJS.messages.error({
        body: '<p> ' + e.responseJSON.message + '</p>'
    });
    console.log(e);
}